require 'spec_helper'

describe 'KumoConfig' do
  let(:config_path) { File.join(File.dirname(__FILE__), 'fixtures') }
  let(:environment_name) { 'production' }

  subject { KumoConfig::EnvironmentConfig.new(
    env_name: environment_name,
    config_path: config_path
  ) }

  it 'resolves the configuration for the specified environment' do
    expect(subject.config['production_key']).to eq('production_value')
  end

  it 'uses commmon configuration if nothing is specified in an environment or injected config' do
    expect(subject.config['common_key']).to eq('common_value')
  end

  it 'gives the environment config precedence over common config' do
    expect(subject.config['overridden_key']).to eq('production override value')
  end

  it 'resolves secret configuration into a Hash' do
    expect(subject.plain_text_secrets['production_secret_key']).to eq('production_secret_value')
  end

  it 'uses commmon secret configuration if nothing is specified in the environment' do
    expect(subject.plain_text_secrets['common_secret_key']).to eq('common_secret_value')
  end

  it 'gives the environment secret precedence over common secrets' do
    expect(subject.plain_text_secrets['overridden_secret']).to eq('production_override_secret')
  end

  context 'when there is injected configuration' do
    subject { KumoConfig::EnvironmentConfig.new(
      env_name: environment_name,
      config_path: config_path,
      injected_config: {
        'injected_key' => 'injected value',
        'overridden_key' => 'injected override value'
      }
    ) }

    it 'resolves injected configuration items' do
      expect(subject.config['injected_key']).to eq('injected value')
    end

    it 'gives injected config precedence over config from files' do
      expect(subject.config['overridden_key']).to eq('injected override value')
    end
  end


  context 'when an environment is specified for which we do not have configuration files' do
    let(:environment_name) { 'nonsense' }

    it 'uses development environment as a default' do
      expect(subject.config['development_key']).to eq('development value')
    end

    it 'uses development secrets' do
      expect(subject.plain_text_secrets['development_secret']).to eq('development_secret_value')
    end
  end
end
